from .fields import (
    DateTimeFieldView,
    NumberFieldView,
    TextFieldView,
    BooleanFieldView,
    ChoicesFieldView,
    ImagesFieldView,
    FileFieldView,
    ItemFieldView
)

from .values import (
    DateTimeValueView,
    NumberValueView,
    TextValueView,
    BooleanValueView,
    ChoicesValueView,
    ImagesValueView,
    FileValueView,
    ChoiceView,
    ImageView,
    FileView,
    ItemValueView
)

from .items import (
    ItemView,
    ItemSchemaView,
    CategoryView
)

from .exports import ExportView, ImportView


from django.core.exceptions import ObjectDoesNotExist
from django.http import QueryDict
from rest_framework import status
from rest_framework.exceptions import NotFound
from rest_framework.response import Response


class RetrieveMixin:

    # noinspection PyUnresolvedReferences
    def retrieve(self, request, pk):
        serializer_context = {'request': request}
        try:
            pk = int(pk)
            instance = self.queryset.get(pk=pk)
        except (ObjectDoesNotExist, ValueError):
            raise NotFound(f'A {self.model_name} with pk={pk} does not exist.')

        serializer = self.serializer_class(
            instance,
            context=serializer_context
        )
        return Response(serializer.data, status=status.HTTP_200_OK)


class ListMixin:

    # noinspection PyUnresolvedReferences
    def list(self, request):
        serializer_context = {'request': request}
        page = self.paginate_queryset(self.get_queryset())

        fields = request.query_params.get('fields', None)
        if fields is not None:
            fields = fields.split(',')

        serializer = self.serializer_class(
            page,
            context=serializer_context,
            many=True,
            fields=fields
        )
        return self.get_paginated_response(serializer.data)


class CreateMixin:

    # noinspection PyUnresolvedReferences
    def create(self, request):
        serializer_context = {'request': request}
        serializer = self.serializer_class(
            data=request.data,
            context=serializer_context
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        serializer = self.serializer_class(
            serializer.instance,
            context=serializer_context
        )
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class UpdateMixin:

    # noinspection PyUnresolvedReferences
    def partial_update(self, request, pk=None):
        try:
            pk = int(pk)
            instance = self.queryset.get(pk=pk)
        except (ObjectDoesNotExist, ValueError):
            raise NotFound(f'A {self.model_name} with pk={pk} does not exist.')

        serializer_context = {'request': request}
        serializer = self.serializer_class(
            instance,
            data=request.data,
            context=serializer_context,
            partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        serializer = self.serializer_class(
            serializer.instance,
            context=serializer_context
        )
        return Response(serializer.data, status=status.HTTP_200_OK)


class DestroyMixin:

    # noinspection PyUnresolvedReferences
    def destroy(self, request, pk=None):
        try:
            pk = int(pk)
            instance = self.queryset.get(pk=pk)
        except (ObjectDoesNotExist, ValueError):
            raise NotFound(f'A {self.model_name} with pk={pk} does not exist.')

        instance.delete()

        return Response(None, status=status.HTTP_204_NO_CONTENT)


# noinspection PyUnresolvedReferences
class FilterMixin:

    multi_query = ()

    def get_queryset(self):
        queryset = self.queryset
        params: QueryDict = self.request.query_params

        if len(params):
            params = params.lists()
            data = {
                key: value if key in self.multi_query else value[0]
                for key, value in params
            }
            serializer_class = self.filter_serializer_class
            serializer = serializer_class(
                data=data,
                context={'request': self.request}
            )
            serializer.is_valid(raise_exception=True)

            params = serializer.data
            order_by = params.pop('order_by', '')
            queryset = queryset.filter(**params)

            if order_by:
                queryset = queryset.order_by(order_by)

        return queryset



# Generated by Django 3.0.2 on 2020-04-23 00:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('drf_schemas', '0002_auto_20200422_1811'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='itemfield',
            name='create_target',
        ),
    ]



# Generated by Django 3.0.2 on 2020-04-22 23:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drf_schemas', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item',
            name='name',
        ),
        migrations.AddField(
            model_name='itemfield',
            name='create_target',
            field=models.BooleanField(blank=True, default=True),
        ),
    ]



import json

from rest_framework import serializers
from . import models


class MaskFieldSerializer(serializers.ModelSerializer):

    def __init__(self, *args, **kwargs):
        # Don't pass the 'fields' arg up to the superclass
        fields = kwargs.pop('fields', None)

        # Instantiate the superclass normally
        super(MaskFieldSerializer, self).__init__(*args, **kwargs)

        if fields is not None:
            # Drop any fields that are not specified in the `fields` argument.
            allowed = set(fields)
            existing = set(self.fields)
            for field_name in existing - allowed:
                self.fields.pop(field_name)


class JSONField(serializers.Field):
    default_error_messages = {
        'invalid': 'Value must be valid JSON.'
    }

    def to_internal_value(self, data):
        try:
            data = json.dumps(data)
        except (TypeError, ValueError):
            self.fail('invalid')
        return data

    def to_representation(self, value):
        try:
            value = json.loads(value)
        except (TypeError, ValueError):
            self.fail('invalid')
        return value


class TrackTimeSerializer:
    created_at = serializers.DateTimeField(
        read_only=True,
        help_text='Date and time when instance was created'
    )
    updated_at = serializers.DateTimeField(
        read_only=True,
        help_text='Date and time when instance was updated'
    )

    class Meta:
        fields = (
            'created_at',
            'updated_at'
        )


class FieldSerializer(MaskFieldSerializer, TrackTimeSerializer):

    # Required
    name = serializers.CharField(
        required=True,
        help_text='Field name'
    )
    item_schema = serializers.PrimaryKeyRelatedField(
        queryset=models.ItemSchema.objects.all(),
        help_text='Schema ID this field belongs to'
    )

    # Optional
    required = serializers.BooleanField(
        required=False,
        default=True,
        help_text='Whether filling this field is mandatory on item creation'
    )
    order = serializers.IntegerField(
        required=False,
        default=0,
        help_text='Display priority of this field'
    )
    help = serializers.CharField(
        required=False,
        allow_blank=True,
        help_text='Help text of this field to show to the user'
    )
    config = JSONField(required=False)

    class Meta:
        fields = TrackTimeSerializer.Meta.fields + (
            'name',
            'required',
            'order',
            'help',
            'item_schema',
            'config',
        )


class ValueSerializer(MaskFieldSerializer, TrackTimeSerializer):

    item = serializers.PrimaryKeyRelatedField(
        queryset=models.Item.objects.all(),
        help_text='Item ID this value belongs to'
    )

    order = serializers.IntegerField(
        read_only=True,
        help_text='Display priority of this value'
    )

    class Meta:
        fields = TrackTimeSerializer.Meta.fields + ('item', 'order')


class DateTimeFieldSerializer(FieldSerializer):

    default = serializers.DateTimeField(
        required=False,
        allow_null=True,
        help_text='Field default value'
    )
    min_value = serializers.DateTimeField(
        required=False,
        allow_null=True,
        help_text='Field minimum value'
    )
    max_value = serializers.DateTimeField(
        required=False,
        allow_null=True,
        help_text='Field maximum value'
    )

    class Meta:
        model = models.DateTimeField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'default',
            'min_value',
            'max_value'
        )


class DateTimeValueSerializer(ValueSerializer):

    value = serializers.DateTimeField(
        required=False,
        allow_null=True,
        help_text='Field value'
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.DateTimeField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.DateTimeValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class NumberFieldSerializer(FieldSerializer):

    default = serializers.FloatField(
        required=False,
        allow_null=True,
        help_text='Field default value'
    )
    min_value = serializers.FloatField(
        required=False,
        allow_null=True,
        help_text='Field minimum value'
    )
    max_value = serializers.FloatField(
        required=False,
        allow_null=True,
        help_text='Field maximum value'
    )

    class Meta:
        model = models.NumberField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'default',
            'min_value',
            'max_value'
        )


class NumberValueSerializer(ValueSerializer):

    value = serializers.FloatField(
        required=False,
        allow_null=True,
        help_text='Field value'
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.NumberField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.NumberValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class TextFieldSerializer(FieldSerializer):

    default = serializers.CharField(
        required=False,
        allow_blank=True,
        help_text='Field default value'
    )

    class Meta:
        model = models.TextField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'default'
        )


class TextValueSerializer(ValueSerializer):

    value = serializers.CharField(
        required=False,
        allow_blank=True,
        help_text='Field value',
        trim_whitespace=False
    )

    field = serializers.PrimaryKeyRelatedField(
        queryset=models.TextField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.TextValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class BooleanFieldSerializer(FieldSerializer):

    default = serializers.BooleanField(
        required=False,
        allow_null=True,
        help_text='Field default value'
    )

    class Meta:
        model = models.BooleanField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'default'
        )


class BooleanValueSerializer(ValueSerializer):

    value = serializers.BooleanField(
        required=False,
        allow_null=True,
        help_text='Field value'
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.BooleanField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.BooleanValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class ChoiceSerializer(MaskFieldSerializer):

    name = serializers.CharField(
        help_text='Choice label'
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.ChoicesField.objects.all(),
        help_text='Field ID this choice belongs to'
    )

    class Meta:
        model = models.Choice
        fields = (
            'id',
            'name',
            'field'
        )


class ChoicesFieldSerializer(FieldSerializer):

    default = serializers.PrimaryKeyRelatedField(
        queryset=models.Choice.objects.all(),
        many=True,
        required=False,
        help_text='Default choices IDs'
    )
    choices = serializers.PrimaryKeyRelatedField(
        queryset=models.Choice.objects.all(),
        many=True,
        required=False,
        help_text='Allowed choices IDs'
    )
    multi = serializers.BooleanField(
        required=False,
        help_text='Whether to allow multi choice selection or not'
    )

    class Meta:
        model = models.ChoicesField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'default',
            'choices',
            'multi'
        )


class ChoicesValueSerializer(ValueSerializer):

    value = serializers.PrimaryKeyRelatedField(
        queryset=models.Choice.objects.all(),
        many=True,
        required=False,
        help_text='Selected choices IDs'
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.ChoicesField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.BooleanValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class ImageSerializer(serializers.ModelSerializer):

    size_bytes = serializers.IntegerField(
        read_only=True,
        help_text='Image size in bytes'
    )
    height = serializers.IntegerField(
        read_only=True,
        help_text='Image height in pixels'
    )
    width = serializers.IntegerField(
        read_only=True,
        help_text='Image width in pixels'
    )

    class Meta:
        model = models.Image
        fields = (
            'id',
            'image',
            'size_bytes',
            'height',
            'width'
        )


class ImagesFieldSerializer(FieldSerializer):

    multi = serializers.BooleanField(
        required=False,
        help_text='Whether to allow multiple image uploads'
    )
    default = serializers.PrimaryKeyRelatedField(
        queryset=models.Image.objects.all(),
        many=True,
        required=False
    )

    class Meta:
        model = models.ImagesField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'multi',
            'default'
        )


class ImagesValueSerializer(ValueSerializer):

    value = serializers.PrimaryKeyRelatedField(
        many=True,
        queryset=models.Image.objects.all(),
        required=False
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.ImagesField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.ImagesValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class FileSerializer(serializers.ModelSerializer, TrackTimeSerializer):

    size_bytes = serializers.IntegerField(
        read_only=True,
        help_text='Image size in bytes'
    )

    class Meta:
        model = models.File
        fields = TrackTimeSerializer.Meta.fields + (
            'id',
            'file',
            'size_bytes'
        )


class FileFieldSerializer(FieldSerializer):

    multi = serializers.BooleanField(
        required=False,
        help_text='Whether to allow multiple file uploads'
    )
    default = serializers.PrimaryKeyRelatedField(
        queryset=models.File.objects.all(),
        many=True,
        required=False
    )

    class Meta:
        model = models.FileField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'multi',
            'default'
        )


class FileValueSerializer(ValueSerializer):

    value = serializers.PrimaryKeyRelatedField(
        many=True,
        queryset=models.File.objects.all(),
        required=False
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.FileField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.FileValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class ItemFieldSerializer(FieldSerializer):

    multi = serializers.BooleanField(
        required=False,
        help_text='Whether to allow multiple related items'
    )

    default = serializers.PrimaryKeyRelatedField(
        queryset=models.Item.objects.all(),
        many=True,
        required=False
    )

    target_schema = serializers.PrimaryKeyRelatedField(
        queryset=models.ItemSchema.objects.all(),
        required=False,
        allow_null=True,
    )

    class Meta:
        model = models.ItemField
        fields = FieldSerializer.Meta.fields + (
            'id',
            'multi',
            'default',
            'target_schema'
        )


class ItemValueSerializer(ValueSerializer):

    value = serializers.PrimaryKeyRelatedField(
        many=True,
        queryset=models.Item.objects.all(),
        required=False
    )
    field = serializers.PrimaryKeyRelatedField(
        queryset=models.ItemField.objects.all(),
        help_text='Field ID this value belongs to'
    )

    class Meta:
        model = models.ItemValue
        fields = ValueSerializer.Meta.fields + (
            'id',
            'value',
            'field'
        )


class CategorySerializer(MaskFieldSerializer, TrackTimeSerializer):
    name = serializers.CharField(
        help_text='Category name'
    )
    description = serializers.CharField(
        help_text='Category description',
        required=False,
        allow_blank=True
    )
    parent = serializers.PrimaryKeyRelatedField(
        queryset=models.Category.objects.all(),
        required=False,
        allow_null=True,
        help_text='Parent category ID'
    )

    class Meta:
        model = models.Category
        fields = TrackTimeSerializer.Meta.fields + (
            'id',
            'name',
            'description',
            'parent'
        )


class ItemSchemaSerializer(MaskFieldSerializer, TrackTimeSerializer):

    name = serializers.CharField(
        help_text='Schema name'
    )

    category = serializers.PrimaryKeyRelatedField(
        queryset=models.Category.objects.all(),
        required=False,
        allow_null=True,
        help_text='Schema category ID'
    )

    image = serializers.PrimaryKeyRelatedField(
        queryset=models.Image.objects.all(),
        required=False,
        allow_null=True,
        help_text='Schema image'
    )
    items_count = serializers.IntegerField(
        read_only=True,
        help_text='Number of items that belongs to this schema'
    )

    config = JSONField(required=False)

    boolean_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of boolean fields of this schema'
    )
    choices_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of choices fields of this schema'
    )
    datetime_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of datetime fields of this schema'
    )
    file_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of file fields of this schema'
    )
    images_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of images fields of this schema'
    )
    item_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of item fields of this schema'
    )
    number_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of number fields of this schema'
    )
    text_fields = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of text fields of this schema'
    )

    class Meta:
        model = models.ItemSchema
        fields = TrackTimeSerializer.Meta.fields + (
            'id',
            'name',
            'category',
            'image',
            'config',
            'items_count',
            'boolean_fields',
            'choices_fields',
            'datetime_fields',
            'file_fields',
            'images_fields',
            'item_fields',
            'number_fields',
            'text_fields',
        )


class ItemSerializer(MaskFieldSerializer, TrackTimeSerializer):

    schema = serializers.PrimaryKeyRelatedField(
        queryset=models.ItemSchema.objects.all(),
        help_text='Item schema ID'
    )

    boolean_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of boolean values of this schema'
    )
    choices_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of choices values of this schema'
    )
    datetime_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of datetime values of this schema'
    )
    file_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of file values of this schema'
    )
    images_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of images values of this schema'
    )
    item_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of item values of this schema'
    )
    number_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of number values of this schema'
    )
    text_values = serializers.PrimaryKeyRelatedField(
        many=True,
        read_only=True,
        help_text='List of text values of this schema'
    )

    class Meta:
        model = models.Item
        fields = TrackTimeSerializer.Meta.fields + (
            'id',
            'schema',
            'boolean_values',
            'choices_values',
            'datetime_values',
            'file_values',
            'images_values',
            'item_values',
            'number_values',
            'text_values',
        )


class FieldFilterSerializer(serializers.Serializer):
    item_schema_id__in = serializers.ListSerializer(
        child=serializers.IntegerField(),
        required=False
    )


class ValueFilterSerializer(serializers.Serializer):
    item_id__in = serializers.ListSerializer(
        child=serializers.IntegerField(),
        required=False
    )


class ChoiceFilterSerializer(serializers.Serializer):
    field_id__in = serializers.ListSerializer(
        child=serializers.IntegerField(),
        required=False
    )
    field__item_schema_id__in = serializers.ListSerializer(
        child=serializers.IntegerField(),
        required=False
    )


class ItemFilterSerializer(serializers.Serializer):

    order_by = serializers.CharField(required=False)

    schema_id = serializers.IntegerField(required=False)

    schema__in = serializers.ListSerializer(
        child=serializers.IntegerField(),
        required=False
    )

    created_at__lte = serializers.DateTimeField(required=False)

    created_at__gte = serializers.DateTimeField(required=False)


class ItemSchemaFilterSerializer(serializers.Serializer):

    order_by = serializers.ChoiceField(
        required=False,
        choices=[
            'name',
            'category',
            'created_at',
            'updated_at',
            '-name',
            '-category',
            '-created_at',
            '-updated_at'
        ]
    )

    name__icontains = serializers.CharField(required=False)

    category_id__in = serializers.ListSerializer(
        child=serializers.IntegerField(),
        required=False
    )

    created_at__lte = serializers.DateTimeField(required=False)

    created_at__gte = serializers.DateTimeField(required=False)


class ImportSerializer(serializers.Serializer):
    file = serializers.FileField()



from rest_framework import viewsets

from ._mixins import (
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    FilterMixin
)
from ..models import (
    DateTimeValue,
    NumberValue,
    TextValue,
    BooleanValue,
    ChoicesValue,
    Choice,
    Image,
    File,
    ImagesValue,
    FileValue,
    ItemValue
)
from ..serializers import (
    DateTimeValueSerializer,
    NumberValueSerializer,
    TextValueSerializer,
    BooleanValueSerializer,
    ChoicesValueSerializer,
    ChoiceSerializer,
    ImagesValueSerializer,
    FileValueSerializer,
    ImageSerializer,
    FileSerializer,
    ItemValueSerializer,
    ChoiceFilterSerializer,
    ValueFilterSerializer
)


class ValueView(
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    FilterMixin,
    viewsets.GenericViewSet
):

    lookup_field = 'pk'
    multi_query = ('item_id__in',)
    filter_serializer_class = ValueFilterSerializer


class DateTimeValueView(ValueView):
    model_name = 'DateTimeValue'
    queryset = DateTimeValue.objects.all()
    serializer_class = DateTimeValueSerializer


class NumberValueView(ValueView):
    model_name = 'NumberValue'
    queryset = NumberValue.objects.all()
    serializer_class = NumberValueSerializer


class TextValueView(ValueView):
    model_name = 'TextValue'
    queryset = TextValue.objects.all()
    serializer_class = TextValueSerializer


class BooleanValueView(ValueView):
    model_name = 'BooleanValue'
    queryset = BooleanValue.objects.all()
    serializer_class = BooleanValueSerializer


class ChoicesValueView(ValueView):
    model_name = 'ChoicesValue'
    queryset = ChoicesValue.objects.all()
    serializer_class = ChoicesValueSerializer


class ImageView(ValueView):
    model_name = 'Image'
    queryset = Image.objects.all()
    serializer_class = ImageSerializer


class FileValueView(ValueView):
    model_name = 'FileValue'
    queryset = FileValue.objects.all()
    serializer_class = FileValueSerializer


class ItemValueView(ValueView):
    model_name = 'ItemValue'
    queryset = ItemValue.objects.all()
    serializer_class = ItemValueSerializer


class ChoiceView(ValueView):
    model_name = 'Choice'
    queryset = Choice.objects.all()
    serializer_class = ChoiceSerializer

    multi_query = ('field_id__in', 'field__item_schema_id__in')
    filter_serializer_class = ChoiceFilterSerializer


class FileView(
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    viewsets.GenericViewSet
):
    model_name = 'File'
    lookup_field = 'pk'
    queryset = File.objects.all()
    serializer_class = FileSerializer

    def get_queryset(self):
        return self.queryset


class ImagesValueView(FileView):
    model_name = 'ImagesValue'
    queryset = ImagesValue.objects.all()
    serializer_class = ImagesValueSerializer



from django.contrib import admin

from . import models


@admin.register(models.DateTimeField)
class DateTimeFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.DateTimeValue)
class DateTimeValueAdmin(admin.ModelAdmin):
    pass


@admin.register(models.NumberField)
class NumberFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.NumberValue)
class NumberValueAdmin(admin.ModelAdmin):
    pass


@admin.register(models.TextField)
class TextFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.TextValue)
class TextValueAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BooleanField)
class BoolFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BooleanValue)
class BooleanValueAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ChoicesField)
class ChoicesFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ChoicesValue)
class ChoicesValueAdmin(admin.ModelAdmin):
    pass


@admin.register(models.Choice)
class ChoiceAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ImagesField)
class ImagesFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ImagesValue)
class ImagesValueFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.FileField)
class FileFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.FileValue)
class FileValueAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ItemField)
class ItemFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ItemValue)
class ItemValueFieldAdmin(admin.ModelAdmin):
    pass


@admin.register(models.ItemSchema)
class ItemSchemaAdmin(admin.ModelAdmin):
    pass


@admin.register(models.Item)
class ItemAdmin(admin.ModelAdmin):
    pass


@admin.register(models.Category)
class CategoryAdmin(admin.ModelAdmin):
    pass


@admin.register(models.Image)
class ImageAdmin(admin.ModelAdmin):
    pass


@admin.register(models.File)
class FileAdmin(admin.ModelAdmin):
    pass




from rest_framework import viewsets

from ._mixins import (
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    FilterMixin
)
from ..models import (
    DateTimeField,
    NumberField,
    TextField,
    BooleanField,
    ChoicesField,
    ImagesField,
    FileField,
    ItemField
)
from ..serializers import (
    DateTimeFieldSerializer,
    NumberFieldSerializer,
    TextFieldSerializer,
    BooleanFieldSerializer,
    ChoicesFieldSerializer,
    ImagesFieldSerializer,
    FileFieldSerializer,
    ItemFieldSerializer,
    FieldFilterSerializer
)


class FieldView(
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    FilterMixin,
    viewsets.GenericViewSet
):

    lookup_field = 'pk'
    multi_query = ('item_schema_id__in',)
    filter_serializer_class = FieldFilterSerializer



class DateTimeFieldView(FieldView):

    model_name = 'DateTimeField'
    queryset = DateTimeField.objects.all()
    serializer_class = DateTimeFieldSerializer


class NumberFieldView(FieldView):

    model_name = 'NumberField'
    queryset = NumberField.objects.all()
    serializer_class = NumberFieldSerializer


class TextFieldView(FieldView):

    model_name = 'TextField'
    queryset = TextField.objects.all()
    serializer_class = TextFieldSerializer


class BooleanFieldView(FieldView):

    model_name = 'BooleanField'
    queryset = BooleanField.objects.all()
    serializer_class = BooleanFieldSerializer


class ChoicesFieldView(FieldView):

    model_name = 'ChoicesField'
    queryset = ChoicesField.objects.all()
    serializer_class = ChoicesFieldSerializer


class ImagesFieldView(FieldView):

    model_name = 'ImagesField'
    queryset = ImagesField.objects.all()
    serializer_class = ImagesFieldSerializer


class FileFieldView(FieldView):

    model_name = 'FileField'
    queryset = FileField.objects.all()
    serializer_class = FileFieldSerializer


class ItemFieldView(FieldView):

    model_name = 'ItemField'
    queryset = ItemField.objects.all()
    serializer_class = ItemFieldSerializer

